/* The following is based on HiSoft C's GSX Function Library */
/* It has been converted from K&R C syntax to modern C syntax */
/* HiSoft's original copyright follows */ 

/*****************************/
/*         HiSoft C          */
/*    GSX Function Library   */
/*       version 1.0         */
/* Copyright (C) 1986 HiSoft */
/* Last changed  26 Apr 1986 */
/* written by David Nutkins  */
/*****************************/

#include "gsx.h"

static uint16_t contrl[12], int_tin[128], ptsin[128], int_tout[128], ptsout[128];

enum
{
	GSX_PB_CONTRL=0,
	GSX_PB_INTIN,
	GSX_PB_PTSIN,
	GSX_PB_INTOUT,
	GSX_PB_PTSOUT,
	
	GSX_PB_MAX,
};

enum
{	
	CONTRL_FUNCTION=0, /* function */
	CONTRL_PTSIN, 
	CONTRL_PTSOUT,	
	CONTRL_INTIN,
	CONTRL_INTOUT,
	CONTRL_SPECIAL
};

enum
{
	CONTROL_FUNCTIONID = 5,
};

enum
{	
	OWS_ID = 0,
	OWS_LINETYPE,
	OWS_POLYLINE_COLOR_INDEX,
	OWS_MARKER_TYPE,
	OWS_POLYMARKER_COLOR_INDEX,
	OWS_TEXT_FONT,
	OWS_TEXT_COLOR_INDEX,
	OWS_FILL_INTERIOR_STYLE,
	OWS_FILL_STYLE_INDEX,
	OWS_FILL_COLOR_INDEX
};

enum
{
	OWS_INTOUT_MAX_WIDTH,
	OWS_INTOUT_MAX_HEIGHT,
	OWS_INTOUT_COORD_FLAGS,
	OWS_INTOUT_PIXEL_WIDTH,
	OWS_INTOUT_PIXEL_HEIGHT
};

static uint16_t * param_list[GSX_PB_MAX] = { contrl,int_tin,ptsin,int_tout,ptsout };

static void gsx(void)
{
	__asm
ld C,#115
ld DE,#_param_list
CALL 5
	__endasm;
}

void v_opnwk(const uint16_t *work_in, uint16_t *handle, uint16_t *work_out) 
{
	const uint16_t *wi = work_in;
	uint16_t i;
	uint16_t *w;
	uint16_t *x;

	x = int_tin;

	for (i = 0; i < 10; ++i) *x++ = *wi++;

	contrl[CONTRL_FUNCTION] = 1;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 11;

	gsx();

	*handle = contrl[6];

	w = work_out; x = int_tout;
	for (i = 0; i < 44; ++i) *w++ = *x++;

	x = ptsout;
	for (; i < 56; ++i) *w++ = *x++;

}

void v_clswk(uint16_t handle)
{
	contrl[CONTRL_FUNCTION] = 2;
	contrl[CONTRL_PTSIN] = contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;
	gsx();
}

void v_clrwk(uint16_t handle)
{
	contrl[CONTRL_FUNCTION] = 3;
	contrl[CONTRL_PTSIN] = contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;
	gsx();
}


void v_updwk(uint16_t handle)
{
	contrl[CONTRL_FUNCTION] = 4;
	contrl[CONTRL_PTSIN] = contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;
	gsx();
}


void v_pline(uint16_t handle, uint16_t count, const uint16_t *pxyarray)
{
	uint16_t i, *x;

	contrl[CONTRL_FUNCTION] = 6;
	contrl[CONTRL_PTSIN] = count;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	x = ptsin;
	for (i = 0; i < count << 1; ++i) *x++ = *pxyarray++;
	gsx();
}


void v_pmarker(uint16_t handle, uint16_t count, const uint16_t *pxyarray)
{
	uint16_t i, *x;

	contrl[CONTRL_FUNCTION] = 7;
	contrl[CONTRL_PTSIN] = count;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	x = ptsin;
	for (i = 0; i < count << 1; ++i) *x++ = *pxyarray++;
	gsx();
}

void v_gtext(uint16_t handle, uint16_t x, uint16_t y, const char *string)
{
	uint16_t *w;

	contrl[CONTRL_FUNCTION] = 8;
	contrl[CONTRL_PTSIN] = 1;
	contrl[6] = handle;

	ptsin[0] = x;
	ptsin[1] = y;

	w = int_tin;
	while (*w++ = *string++);

	contrl[CONTRL_INTIN] = --w - int_tin;

	gsx();
}

void v_fillarea(uint16_t handle, uint16_t count, const uint16_t *pxyarray)
{
	uint16_t i, *x;

	contrl[CONTRL_FUNCTION] = 9;
	contrl[CONTRL_PTSIN] = count;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	x = ptsin;
	for (i = 0; i < count << 1; ++i) *x++ = *pxyarray++;
	gsx();
}

void v_bar(uint16_t handle, const uint16_t *pxyarray)
{
	contrl[CONTRL_FUNCTION] = 11;
	contrl[CONTRL_PTSIN] = 2;
	contrl[CONTRL_INTIN] = 0;
	contrl[CONTROL_FUNCTIONID] = 1;
	contrl[6] = handle;

	ptsin[0] = pxyarray[0];
	ptsin[1] = pxyarray[1];
	ptsin[2] = pxyarray[2];
	ptsin[3] = pxyarray[3];

	gsx();
}


void vst_height(uint16_t handle, uint16_t height, uint16_t *char_width, uint16_t *char_height, uint16_t *cell_width, uint16_t *cell_height)
{

	contrl[CONTRL_FUNCTION] = 12;
	contrl[CONTRL_PTSIN] = 1;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	ptsin[0] = 0;
	ptsin[1] = height;

	gsx();

	*char_width = ptsout[0];
	*char_height = ptsout[1];
	*cell_width = ptsout[2];
	*cell_height = ptsout[3];
}


uint16_t vst_rotation(uint16_t handle, uint16_t angle)
{
	contrl[CONTRL_FUNCTION] = 13;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 1;
	contrl[6] = handle;

	int_tin[0] = angle;

	gsx();

	return int_tout[0];
}

void vs_color(uint16_t handle, uint16_t index, const uint16_t *rgb_in)
{
	contrl[CONTRL_FUNCTION] = 14;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 4;
	contrl[6] = handle;

	int_tin[0] = index;
	int_tin[1] = rgb_in[0];
	int_tin[2] = rgb_in[1];
	int_tin[3] = rgb_in[2];

	gsx();

}


uint16_t vsl_type(uint16_t handle, uint16_t style)
{
	return vs_style(15, handle, style);
}

uint16_t vsl_color(uint16_t handle, uint16_t color_index)
{
	return vs_style(17, handle, color_index);
}

uint16_t vsm_type(uint16_t handle, uint16_t symbol)
{
	return vs_style(18, handle, symbol);
}

uint16_t vsm_height(uint16_t handle, uint16_t height)
{
	contrl[CONTRL_FUNCTION] = 19;
	contrl[CONTRL_PTSIN] = 1;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	ptsin[0] = 0;
	ptsin[1] = height;

	gsx();

	return ptsout[1];
}

uint16_t vsm_color(uint16_t handle, uint16_t color_index)
{
	return vs_style(20, handle, color_index);
}

uint16_t vst_color(uint16_t handle, uint16_t color_index)
{
	return vs_style(22, handle, color_index);
}

uint16_t vsf_interior(uint16_t handle, uint16_t style)
{
	return vs_style(23, handle, style);
}

uint16_t vsf_style(uint16_t handle, uint16_t style_index)
{
	return vs_style(24, handle, style_index);
}

uint16_t vsf_color(uint16_t handle, uint16_t color_index)
{
	return vs_style(25, handle, color_index);
}

void vrq_locator(uint16_t handle, uint16_t x, uint16_t y, uint16_t *xout, uint16_t *yout, uint16_t *term)
{
	contrl[CONTRL_FUNCTION] = 28;
	contrl[CONTRL_PTSIN] = 1;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	ptsin[0] = x;
	ptsin[1] = y;

	gsx();

	*xout = ptsout[0];
	*yout = ptsout[1];
	*term = int_tout[0];

}


uint16_t vsm_locator(uint16_t handle, uint16_t x, uint16_t y, uint16_t *xout, uint16_t *yout, uint16_t *term)
{
	contrl[CONTRL_FUNCTION] = 28;
	contrl[CONTRL_PTSIN] = 1;
	contrl[CONTRL_INTIN] = 0;
	contrl[6] = handle;

	ptsin[0] = x;
	ptsin[1] = y;

	gsx();

	*xout = ptsout[0];
	*yout = ptsout[1];
	*term = int_tout[0];
	
	return contrl[CONTRL_PTSOUT] 	| (contrl[CONTRL_INTOUT] << 1);
}

void vsin_mode(uint16_t handle, uint16_t dev_type, uint16_t mode)
{

	contrl[CONTRL_FUNCTION] = 33;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 2;
	contrl[6] = handle;

	int_tin[0] = dev_type;
	int_tin[1] = mode;

	gsx();

}

void vq_chcells(uint16_t handle, uint16_t *rows, uint16_t *columns)
{
	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 0;
	contrl[CONTROL_FUNCTIONID] = 1;
	contrl[6] = handle;

	gsx();

	*rows = int_tout[0];
	*columns = int_tout[1];
}

void v_exit_cur(uint16_t handle)

{
	v_escape(handle, 2);
}

void v_enter_cur(uint16_t handle)

{
	v_escape(handle, 3);
}

void v_curup(uint16_t handle)

{
	v_escape(handle, 4);
}

void v_curdown(uint16_t handle)

{
	v_escape(handle, 5);
}

void v_curright(uint16_t handle)

{
	v_escape(handle, 6);
}

void v_curleft(uint16_t handle)

{
	v_escape(handle, 7);
}

void v_curhome(uint16_t handle)

{
	v_escape(handle, 8);
}

void v_eeos(uint16_t handle)

{
	v_escape(handle, 9);
}

void v_eeol(uint16_t handle)

{
	v_escape(handle, 10);
}

void vs_curaddress(uint16_t handle, uint16_t row, uint16_t column)
{
	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 2;
	contrl[CONTROL_FUNCTIONID] = 11;
	contrl[6] = handle;

	int_tin[0] = row;
	int_tin[1] = column;
	gsx();

}

void vs_curtext(uint16_t handle, const char *string)
{
	uint16_t *w;

	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTROL_FUNCTIONID] = 12;
	contrl[6] = handle;

	w = int_tin;
	while (*w++ = *string++);

	contrl[CONTRL_INTIN] = --w - int_tin;

	gsx();
}


void v_rvon(uint16_t handle)

{
	v_escape(handle, 13);
}

void v_rvoff(uint16_t handle)

{
	v_escape(handle, 14);
}

void vq_curaddress(uint16_t handle, uint16_t *row, uint16_t *column)
{
	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 0;
	contrl[CONTROL_FUNCTIONID] = 15;
	contrl[6] = handle;

	gsx();

	*row = int_tout[0];
	*column = int_tout[1];
}

void v_dspcur(uint16_t handle, uint16_t x, uint16_t y)
{
	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 1;
	contrl[CONTRL_INTIN] = 0;
	contrl[CONTROL_FUNCTIONID] = 18;
	contrl[6] = handle;

	ptsin[0] = x;
	ptsin[1] = y;

	gsx();
}

void v_rmcur(uint16_t handle)
{
	v_escape(handle, 19);
}


/* only normally called internally*/
uint16_t vs_style(uint16_t opcode, uint16_t handle, uint16_t style)
{
	contrl[CONTRL_FUNCTION] = opcode;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 1;
	contrl[6] = handle;

	int_tin[0] = style;

	gsx();

	return int_tout[0];
}

/* only normally called uint16_ternally*/
void v_escape(uint16_t handle, uint16_t function)
{
	contrl[CONTRL_FUNCTION] = 5;
	contrl[CONTRL_PTSIN] = 0;
	contrl[CONTRL_INTIN] = 0;
	contrl[CONTROL_FUNCTIONID] = function;
	contrl[6] = handle;

	gsx();

}



/*****************************/
/*         Hisoft C          */
/*   GSX Function Library    */
/*           End             */
/*****************************/
